@@ -79,8 +79,7 @@ describe Rack::OAuth2::Client do
   describe '#access_token!' do
     subject { client.access_token! }
 
-      context 'when *args given' do
-
+    context '*args handling' do
       describe 'client authentication method' do
         before do
           client.authorization_code = 'code'
@@ -98,6 +97,108 @@ describe Rack::OAuth2::Client do
           client.access_token!
         end
 
+        context 'when jwt_bearer auth method specified' do
+          context 'when client_secret is given' do
+            it 'should be JWT bearer client assertion w/ auto-generated HS256-signed JWT assertion' do
+              mock_response(
+                :post,
+                'https://server.example.com/oauth2/token',
+                'tokens/bearer.json',
+                params: {
+                  client_assertion: /^eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9\..+/, # NOTE: HS256
+                  client_assertion_type: Rack::OAuth2::URN::ClientAssertionType::JWT_BEARER,
+                  code: 'code',
+                  grant_type: 'authorization_code',
+                  redirect_uri: 'https://client.example.com/callback'
+                }
+              )
+              client.access_token! :jwt_bearer
+            end
+          end
+
+          context 'when private_key is given' do
+            context 'when RSA key' do
+              let :client do
+                Rack::OAuth2::Client.new(
+                  identifier: 'client_id',
+                  private_key: OpenSSL::PKey::RSA.generate(2048),
+                  host: 'server.example.com',
+                  redirect_uri: 'https://client.example.com/callback'
+                )
+              end
+
+              it 'should be JWT bearer client assertion w/ auto-generated RS256-signed JWT assertion' do
+                mock_response(
+                  :post,
+                  'https://server.example.com/oauth2/token',
+                  'tokens/bearer.json',
+                  params: {
+                    client_assertion: /^eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9\..+/, # NOTE: RS256
+                    client_assertion_type: Rack::OAuth2::URN::ClientAssertionType::JWT_BEARER,
+                    code: 'code',
+                    grant_type: 'authorization_code',
+                    redirect_uri: 'https://client.example.com/callback'
+                  }
+                )
+                client.access_token! :jwt_bearer
+              end
+            end
+
+            context 'when EC key' do
+              let :client do
+                Rack::OAuth2::Client.new(
+                  identifier: 'client_id',
+                  private_key: OpenSSL::PKey::EC.new('prime256v1').generate_key,
+                  host: 'server.example.com',
+                  redirect_uri: 'https://client.example.com/callback'
+                )
+              end
+
+              it 'should be JWT bearer client assertion w/ auto-generated ES256-signed JWT assertion' do
+                mock_response(
+                  :post,
+                  'https://server.example.com/oauth2/token',
+                  'tokens/bearer.json',
+                  params: {
+                    client_assertion: /^eyJ0eXAiOiJKV1QiLCJhbGciOiJFUzI1NiJ9\..+/, # NOTE: ES256
+                    client_assertion_type: Rack::OAuth2::URN::ClientAssertionType::JWT_BEARER,
+                    code: 'code',
+                    grant_type: 'authorization_code',
+                    redirect_uri: 'https://client.example.com/callback'
+                  }
+                )
+                client.access_token! :jwt_bearer
+              end
+            end
+          end
+
+          context 'when client_assertion is explicitly given' do
+            let :client do
+              Rack::OAuth2::Client.new(
+                identifier: 'client_id',
+                host: 'server.example.com',
+                redirect_uri: 'https://client.example.com/callback'
+              )
+            end
+
+            it 'should be JWT bearer client assertion w/ specified assertion' do
+              mock_response(
+                :post,
+                'https://server.example.com/oauth2/token',
+                'tokens/bearer.json',
+                params: {
+                  client_assertion: 'any.jwt.assertion',
+                  client_assertion_type: Rack::OAuth2::URN::ClientAssertionType::JWT_BEARER,
+                  code: 'code',
+                  grant_type: 'authorization_code',
+                  redirect_uri: 'https://client.example.com/callback'
+                }
+              )
+              client.access_token! :jwt_bearer, client_assertion: 'any.jwt.assertion'
+            end
+          end
+        end
+
         context 'when other auth method specified' do
           it 'should be body params' do
             mock_response(
@@ -169,7 +270,7 @@ describe Rack::OAuth2::Client do
     end
 
     context 'when bearer token is given' do
-      before  do
+      before do
         client.authorization_code = 'code'
         mock_response(
           :post,
@@ -184,7 +285,7 @@ describe Rack::OAuth2::Client do
       its(:expires_in) { should == 3600 }
 
       context 'when token type is "Bearer", not "bearer"' do
-        before  do
+        before do
           client.authorization_code = 'code'
           mock_response(
             :post,
@@ -198,7 +299,7 @@ describe Rack::OAuth2::Client do
     end
 
     context 'when mac token is given' do
-      before  do
+      before do
         client.authorization_code = 'code'
         mock_response(
           :post,
@@ -214,7 +315,7 @@ describe Rack::OAuth2::Client do
     end
 
     context 'when no-type token is given (JSON)' do
-      before  do
+      before do
         client.authorization_code = 'code'
         mock_response(
           :post,
@@ -227,6 +328,14 @@ describe Rack::OAuth2::Client do
       its(:access_token) { should == 'access_token' }
       its(:refresh_token) { should == 'refresh_token' }
       its(:expires_in) { should == 3600 }
+
+      context 'when token_type is forced' do
+        before do
+          client.force_token_type! :bearer
+        end
+        it { should be_instance_of Rack::OAuth2::AccessToken::Bearer }
+        its(:token_type) { should == :bearer }
+      end
     end
 
     context 'when no-type token is given (key-value)' do
@@ -255,7 +364,7 @@ describe Rack::OAuth2::Client do
     end
 
     context 'when unknown-type token is given' do
-      before  do
+      before do
         client.authorization_code = 'code'
         mock_response(
           :post,
